@@ -1,8 +1,7 @@
 #!/usr/bin/perl
 
-# $Id: Peer.pm,v 1.15 2003/10/28 09:06:59 unimlo Exp $
-
 package Net::BGP::Peer;
+use bytes;
 
 use strict;
 use vars qw(
@@ -13,7 +12,7 @@ use vars qw(
 ## Inheritance and Versioning ##
 
 @ISA     = qw( Exporter );
-$VERSION = '0.07';
+$VERSION = '0.16';
 
 ## General Definitions ##
 
@@ -104,8 +103,12 @@ sub new
         _transport             => undef,
         _listen                => TRUE,
         _passive               => FALSE,
-        _refresh               => FALSE,
+        _announce_refresh      => FALSE,
+        _support_capabilities  => TRUE,
+        _support_mbgp          => TRUE,
+        _support_as4           => FALSE,
         _open_callback         => undef,
+        _established_callback  => undef,
         _keepalive_callback    => undef,
         _update_callback       => undef,
         _refresh_callback      => undef,
@@ -120,55 +123,74 @@ sub new
     while ( defined($arg = shift()) ) {
         $value = shift();
 
-        if (( $arg =~ /start/i )
-         || ( $arg =~ /connectretrytime/i )
-         || ( $arg =~ /keepalivetime/i )
-         || ( $arg =~ /holdtime/i )) {
+        if (( $arg =~ /^start$/i )
+         || ( $arg =~ /^connectretrytime$/i )
+         || ( $arg =~ /^keepalivetime$/i )
+         || ( $arg =~ /^holdtime$/i )) {
             $transarg{$arg} = $value;
         }
-        elsif ( $arg =~ /thisid/i ) {
+        elsif ( $arg =~ /^thisid$/i ) {
             $this->{_local_id} = $value;
         }
-        elsif ( $arg =~ /thisas/i ) {
+        elsif ( $arg =~ /^thisas$/i ) {
             $this->{_local_as} = $value;
         }
-        elsif ( $arg =~ /peerid/i ) {
+        elsif ( $arg =~ /^peerid$/i ) {
             $this->{_peer_id} = $value;
         }
-        elsif ( $arg =~ /peeras/i ) {
+        elsif ( $arg =~ /^peeras$/i ) {
             $this->{_peer_as} = $value;
         }
-        elsif ( $arg =~ /peerport/i ) {
+        elsif ( $arg =~ /^peerport$/i ) {
             $this->{_peer_port} = $value;
         }
-        elsif ( $arg =~ /listen/i ) {
+        elsif ( $arg =~ /^listen$/i ) {
             $this->{_listen} = $value;
         }
-        elsif ( $arg =~ /passive/i ) {
+        elsif ( $arg =~ /^passive$/i ) {
             $this->{_passive} = $value;
         }
-        elsif ( $arg =~ /refresh/i ) {
-            $this->{_refresh} = $value;
-        }
-        elsif ( $arg =~ /opencallback/i ) {
-            $this->{_open_callback} = $value;
-        }
-        elsif ( $arg =~ /keepalivecallback/i ) {
-            $this->{_keepalive_callback} = $value;
-        }
-        elsif ( $arg =~ /updatecallback/i ) {
-            $this->{_update_callback} = $value;
-        }
-        elsif ( $arg =~ /refreshcallback/i ) {
+        elsif ( $arg =~ /^refreshcallback$/i ) {
             $this->{_refresh_callback} = $value;
         }
-        elsif ( $arg =~ /resetcallback/i ) {
+        elsif ( $arg =~ /^refresh$/i ) {
+            warnings::warnif(
+                'deprecated',
+                'Refresh in is deprecated, use AnnounceRefresh instead'
+            );
+            $this->{_announce_refresh} = $value;
+        }
+        elsif ( $arg =~ /^announcerefresh$/i ) {
+            $this->{_announce_refresh} = $value;
+        }
+        elsif ( $arg =~ /^supportcapabilities$/i ) {
+            $this->{_support_capabilities} = $value;
+        }
+        elsif ( $arg =~ /^supportmbgp$/i ) {
+            $this->{_support_mbgp} = $value;
+        }
+        elsif ( $arg =~ /^supportas4$/i ) {
+            $this->{_support_as4} = $value;
+        }
+        elsif ( $arg =~ /^opencallback$/i ) {
+            $this->{_open_callback} = $value;
+        }
+        elsif ( $arg =~ /^establishedcallback$/i ) {
+            $this->{_established_callback} = $value;
+        }
+        elsif ( $arg =~ /^keepalivecallback$/i ) {
+            $this->{_keepalive_callback} = $value;
+        }
+        elsif ( $arg =~ /^updatecallback$/i ) {
+            $this->{_update_callback} = $value;
+        }
+        elsif ( $arg =~ /^resetcallback$/i ) {
             $this->{_reset_callback} = $value;
         }
-        elsif ( $arg =~ /notificationcallback/i ) {
+        elsif ( $arg =~ /^notificationcallback$/i ) {
             $this->{_notification_callback} = $value;
         }
-        elsif ( $arg =~ /errorcallback/i ) {
+        elsif ( $arg =~ /^errorcallback$/i ) {
             $this->{_error_callback} = $value;
         }
         else {
@@ -177,7 +199,7 @@ sub new
     }
 
     $transarg{parent} = $this;
-    $this->{_transport} = new Net::BGP::Transport(%transarg);
+    $this->{_transport} = Net::BGP::Transport->new(%transarg);
 
     weaken($knownpeers{$this} = $this);
     return ( $this );
@@ -255,7 +277,25 @@ sub this_as
 sub this_can_refresh
 {
     my $this = shift();
-    return ( $this->{_refresh} );
+    return ( $this->{_announce_refresh} );
+}
+
+sub this_can_as4
+{
+    my $this = shift();
+    return ( $this->{_support_as4} );
+}
+
+sub support_capabilities
+{
+    my $this = shift();
+    return ( $this->{_support_capabilities} );
+}
+
+sub support_mbgp
+{
+    my $this = shift();
+    return ( $this->{_support_mbgp} );
 }
 
 sub peer_id
@@ -276,6 +316,18 @@ sub peer_port
     return ( $this->{_peer_port} );
 }
 
+sub peer_can_as4
+{
+    my $this = shift();
+    return $this->transport->can_as4;
+}
+
+sub peer_can_mbgp
+{
+    my $this = shift();
+    return $this->transport->can_mbgp;
+}
+
 sub peer_can_refresh
 {
     my $this = shift();
@@ -306,6 +358,12 @@ sub set_open_callback
     $this->{_open_callback} = $callback;
 }
 
+sub set_established_callback
+{
+    my ($this, $callback) = @_;
+    $this->{_established_callback} = $callback;
+}
+
 sub set_keepalive_callback
 {
     my ($this, $callback) = @_;
@@ -391,6 +449,15 @@ sub open_callback
     }
 }
 
+sub established_callback
+{
+    my $this = shift();
+
+    if ( defined($this->{_established_callback}) ) {
+        &{ $this->{_established_callback} }($this);
+    }
+}
+
 sub keepalive_callback
 {
     my $this = shift();
@@ -487,7 +554,7 @@ Net::BGP::Peer - Class encapsulating BGP-4 peering session state and functionali
 
     use Net::BGP::Peer;
 
-    $peer = new Net::BGP::Peer(
+    $peer = Net::BGP::Peer->new(
         Start                => 1,
         ThisID               => '10.0.0.1',
         ThisAS               => 64512,
@@ -499,6 +566,10 @@ Net::BGP::Peer - Class encapsulating BGP-4 peering session state and functionali
         KeepAliveTime        => 20,
         Listen               => 0,
         Passive              => 0,
+        AnnounceRefresh      => 1,
+        SupportCapabilities  => 1,
+        SupportMBGP          => 1,
+        SupportAS4           => 1,
         OpenCallback         => \&my_open_callback,
         KeepaliveCallback    => \&my_keepalive_callback,
         UpdateCallback       => \&my_update_callback,
@@ -521,9 +592,15 @@ Net::BGP::Peer - Class encapsulating BGP-4 peering session state and functionali
     $peer_id = $peer->peer_id();
     $peer_as = $peer->peer_as();
 
+    $i_will  = $peer->support_capabilities();
+
+    $i_mbgp  = $peer->support_mbgp();
+
     $i_can   = $peer->this_can_refresh();
     $peer_can= $peer->peer_can_refresh();
 
+    $peer_as4= $peer->peer_can_as4();
+
     $listen  = $peer->is_listener();
     $passive = $peer->is_passive();
     $estab   = $peer->is_established();
@@ -534,6 +611,7 @@ Net::BGP::Peer - Class encapsulating BGP-4 peering session state and functionali
     $string  = $peer->asstring();
 
     $peer->set_open_callback(\&my_open_callback);
+    $peer->set_established_callback(\&my_established_callback);
     $peer->set_keepalive_callback(\&my_keepalive_callback);
     $peer->set_update_callback(\&my_update_callback);
     $peer->set_notification_callback(\&my_notification_callback);
@@ -638,7 +716,32 @@ connections.
 
 This parameter specifies whether the B<Net::BGP::Peer> will annonce support
 for route refresh ('soft re-configure' as specified by RFC 2918). No support
-for route refresh is implemented - only the B<RefreshCallback> function.
+for route refresh is implemented - only the B<RefreshCallback> function.  This
+has no effect if SupportCapabilities is FALSE.
+
+=head2 SupportCapabilities
+
+This parameter specifies whether the B<Net::BGP::Peer> will attempt to
+negotiate capabilities.  You can set this to FALSE if talking to an old BGP
+speaker that doesn't support it (you'll get a notification message for an
+unsupported capability if this is the case).  This defaults to TRUE.
+
+=head2 SupportMBGP
+
+This parameter specifies whether the B<NET::BGP::Peer> will attempt to
+negotiate MBGP.  Quagga (and probably others) need this if you want to send
+the REFRESH capability. Today this just indicates support for IPv4 Unicast.
+This defaults to TRUE.  This has no effect if SupportCapabilities is FALSE.
+
+=head2 SupportAS4
+
+This paramemter specifies whether outgoing connections from B<NET::BGP::Peer>
+will attempt to negotiate AS4 (32 bit ASNs). For received connections, this
+parameter has no effect - it only determines whether or not AS4 is negotiated
+during outgoing connection.  For received connections, this will be changed
+to TRUE (on the listening connection) whenever the appropriate OPEN capability
+is received.  Note that the B<SupportCapabilities> must be true for this to
+be sent.  This defaults to FALSE.
 
 =head2 OpenCallback
 
@@ -749,6 +852,10 @@ I<peer_as()>
 
 I<this_can_refresh()>
 
+I<support_capabilities()>
+
+I<support_mbgp()>
+
 I<is_listener()>
 
 I<is_passive()>
@@ -776,6 +883,8 @@ IP and AS numbers.
 
 I<set_open_callback()>
 
+I<set_established_callback()>
+
 I<set_keepalive_callback()>
 
 I<set_update_callback()>
@@ -789,7 +898,8 @@ I<set_notification_callback()>
 I<set_error_callback()>
 
 These methods set the callback functions which are invoked whenever the
-peer receives the corresponding BGP message type from its peer. They
+peer receives the corresponding BGP message type from its peer, or, in the
+case of I<set_established_callback>, transitions to the relevant state. They
 can be set in the constructor as well as with these methods. These methods
 each take one argument, which is the subroutine reference to be invoked.
 A callback function can be removed by calling the corresponding one of these
@@ -845,11 +955,11 @@ in the UPDATE or REFRESH message, while in the latter two cases it is a
 B<Net::BGP::Notification> object encapsulating the information in the
 NOTIFICATION message sent or received.
 
-The RESET callback is special, since it is used whenever an established BGP
-session is reset, even though no message has been recieved or sent. The REFRESH
-callback is also special, since it is also called without a REFRESH object
-whenever a BGP session is established. The two callbacks can be used to clear
-and retransmit a RIB from/to the peer in question.
+The RESET and ESTABLISHED callbacks are special, since they are used whenever an
+established BGP session is reset, even though no message has been recieved or sent.
+The REFRESH callback is also special, since it is also called without a REFRESH
+object whenever a BGP session is established. The two callbacks can be used to
+clear and retransmit a RIB from/to the peer in question.
 
 Whenever a callback function is to be invoked, the action occuring internally is
 the invocation of one of the following methods, corresponding to the event which
@@ -857,6 +967,8 @@ has occured:
 
 I<open_callback()>
 
+I<established_callback()>
+
 I<keepalive_callback()>
 
 I<update_callback()>
@@ -872,9 +984,9 @@ I<error_callback()>
 Internally, each of these methods just checks to see whether a program defined
 callback function has been set and calls it if so, passing it arguments as
 described above. As an alternative to providing subroutine references to the
-constructor or through the I<set_open_callback()>, I<set_keepalive_callback()>,
-I<set_update_callback()>, I<set_refresh_callback()>, I<set_reset_callback()>,
-I<set_notification_callback()>, and I<set_error_callback()>
+constructor or through the I<set_open_callback()>, I<set_established_callback()>,
+I<set_keepalive_callback()>, I<set_update_callback()>, I<set_refresh_callback()>,
+I<set_reset_callback()>, I<set_notification_callback()>, and I<set_error_callback()>
 methods, an application may effect a similar result by sub-classing the
 B<Net::BGP::Peer> module and overridding the defintions of the above methods
 to perform whatever actions would have been executed by ordinary callback functions.
